from random import randint
from math import sin, cos, atan2, radians, sqrt

from pygame import Surface
from pygame.locals import *

def get_step(start, end, speed):
    x0, y0 = start
    x1, y1 = end
    angle = atan2(x1 - x0, y1 - y0)
    return speed * sin(angle), speed * cos(angle)

def get_endpoint(start, angle, magnitude):
    """clockwise, 0 is up"""
    x0, y0 = start
    dx, dy = get_delta(angle, magnitude)
    return x0 + dx, y0 + dy

def get_delta(angle, magnitude):
    angle = radians(angle)
    return sin(angle) * magnitude, -cos(angle) * magnitude

def rotate_2d(point, center, angle, translate_angle=True):
    if translate_angle:
        angle = radians(angle)
    x, y = point
    cx, cy = center
    return cos(angle) * (x - cx) - sin(angle) * (y - cy) + cx, \
           sin(angle) * (x - cx) + cos(angle) * (y - cy) + cy

def get_points_on_circle(center, radius, count, offset=0):
    angle_step = 360.0 / count
    points = []
    current_angle = 0
    for _ in xrange(count):
        points.append(get_point_on_circle(center, radius,
                                          current_angle + offset))
        current_angle += angle_step
    return points

def get_point_on_circle(center, radius, angle, translate_angle=True):
    if translate_angle:
        angle = radians(angle)
    return center[0] + sin(angle) * radius, center[1] - cos(angle) * radius

def get_range_steps(start, end, count):
    for ii in xrange(count):
        yield start + (end - start) * ii / float(count - 1)

def get_distance(p0, p1):
    return sqrt((p0[0] - p1[0]) ** 2 + (p0[1] - p1[1]) ** 2)

def place_in_rect(rect, incoming, contain=True, *args):
    while True:
        incoming.center = randint(0, rect.w), randint(0, rect.h)
        if not contain or rect.contains(incoming):
            collides = False
            for inner in args:
                if inner.colliderect(incoming):
                    collides = True
                    break
            if not collides:
                break

# from http://www.realtimerendering.com/resources/GraphicsGems/gemsii/xlines.c
def get_intersection(p0, p1, p2, p3):
    x0, y0 = p0
    x1, y1 = p1
    x2, y2 = p2
    x3, y3 = p3
    a0 = y1 - y0
    b0 = x0 - x1
    c0 = x1 * y0 - x0 * y1
    r2 = a0 * x2 + b0 * y2 + c0
    r3 = a0 * x3 + b0 * y3 + c0
    if r2 != 0 and r3 != 0 and r2 * r3 > 0:
        return None
    a1 = y3 - y2
    b1 = x2 - x3
    c1 = x3 * y2 - x2 * y3
    r0 = a1 * x0 + b1 * y0 + c1
    r1 = a1 * x1 + b1 * y1 + c1
    if r0 != 0 and r1 != 0 and r0 * r1 > 0:
        return None
    denominator = a0 * b1 - a1 * b0
    if denominator == 0:
        return (x0 + x1 + x2 + x3) / 4, (y0 + y1 + y2 + y3) / 4
    if denominator < 0:
        offset = -denominator / 2
    else:
        offset = denominator / 2
    numerator = b0 * c1 - b1 * c0
    x = ((-1, 1)[numerator < 0] * offset + numerator) / denominator
    numerator = a1 * c0 - a0 * c1
    y = ((-1, 1)[numerator < 0] * offset + numerator) / denominator
    return x, y

def collide_line_with_rect(rect, p0, p1):
    for line in ((rect.topleft, rect.topright),
                 (rect.topright, rect.bottomright),
                 (rect.bottomright, rect.bottomleft),
                 (rect.bottomleft, rect.topleft)):
        if get_intersection(p0, p1, *line):
            return True

def render_box(font, text, antialias, color, background=None, border=None,
               border_width=1, padding=0):
    surface = font.render(text, antialias, color, background)
    if padding:
        if isinstance(padding, int):
            padding = [padding] * 2
        padding = [x * 2 for x in padding]
        rect = surface.get_rect()
        padded_surface = Surface(rect.inflate(padding).size, SRCALPHA)
        if background is not None:
            padded_surface.fill(background)
        rect.center = padded_surface.get_rect().center
        padded_surface.blit(surface, rect)
        surface = padded_surface
    if border is not None:
        if isinstance(border_width, int):
            border_width = [border_width] * 2
        border_width = [x * 2 for x in border_width]
        rect = surface.get_rect()
        bordered_surface = Surface(rect.inflate(border_width).size)
        bordered_surface.fill(border)
        rect.center = bordered_surface.get_rect().center
        bordered_surface.blit(surface, rect)
        surface = bordered_surface
    return surface
